﻿using System;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;
using ThinkGeo.MapSuite.MapSuiteGeocoder;

namespace GeocoderThreadTest
{
    public partial class Form1 : Form
    {
        Collection<string> testDataRecords = new Collection<string>();

        event EventHandler<EventArgs> MyEvent;
        int completedThreadCount = 0;

        public Form1()
        {
            InitializeComponent();
            string dataPath = ConfigurationManager.AppSettings["GeocoderDataPath"].ToString();
            MyEvent += new EventHandler<EventArgs>(Form1_MyEvent);
        }

        void Form1_MyEvent(object sender, EventArgs e)
        {
            completedThreadCount++;
            Trace.WriteLine("Geocoder Completed Thread Count: " + completedThreadCount);
            Trace.WriteLine("Geocoder Current Thread ManagedThreadId: " + Thread.CurrentThread.ManagedThreadId);
        }

        private void btnMultiInstances_Click(object sender, EventArgs e)
        {
            Trace.WriteLine(string.Format("Geocoder: {0} Threads Start!", (int)(threadCount.Value)));

            for (int i = 0; i < (int)(threadCount.Value); i++)
            {
                string exePath = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) + @"\TestData\target\";
                Thread thread = new Thread(new ParameterizedThreadStart(GeocodeTestData));
                thread.Name = "Geocoder" + i.ToString();
                thread.Start(new object[] { i, (int)(threadCount.Value) });
            }
        }

        private void GeocodeTestData(object parametersInObject)
        {
            Stopwatch stopWatch = new Stopwatch();
            stopWatch.Start();

            object[] parameters = (object[])parametersInObject;
            int threadNumber = (int)parameters[0];
            int threadCount = (int)parameters[1];

            string dataPath = ConfigurationManager.AppSettings["GeocoderDataPath"].ToString();
            UsaGeocoder usaGeocoder = new UsaGeocoder(dataPath);
            usaGeocoder.Open();

            for (int i = 0; i < testDataRecords.Count; i++)
            {
                string item = testDataRecords[i];
                if (i % threadCount != threadNumber)
                {
                    continue;
                }
                string[] parts = item.Split(',');
                string address = string.Format("{0} {1} {2} {3} {4}", parts[5], parts[1], parts[2], parts[3], parts[4]);

                if (string.IsNullOrEmpty(address) || (parts[1] == "NULL"))
                {
                    continue;
                }
                Collection<GeocoderMatch> result = usaGeocoder.Match(address, parts[8]);
            }

            stopWatch.Stop();
            Trace.WriteLine(string.Format("Geocoder: Thread {0}, Time: {1}s", threadNumber, stopWatch.ElapsedMilliseconds / 1000));
            usaGeocoder.Close();
            GC.Collect();
            GC.SuppressFinalize(this);
            GC.Collect();

            MyEvent(this, new EventArgs());
            
        }

        private void Reverse_Click(object sender, EventArgs e)
        {
            Trace.WriteLine(string.Format("Revearse Geocoder: {0} Threads Start!", (int)(threadCount.Value)));

            for (int i = 0; i < (int)(threadCount.Value); i++)
            {
                string exePath = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) + @"\FennieMae\target\";

                Thread thread = new Thread(new ParameterizedThreadStart(ReverseGeocodeTestData));
                thread.Name = "Geocoder" + i.ToString();
                thread.Start(new object[] { i, (int)threadCount.Value });
            }
        }

        private void ReverseGeocodeTestData(object parametersInObject)
        {
            Stopwatch stopWatch = new Stopwatch();
            stopWatch.Start();
            object[] parameters = (object[])parametersInObject;
            int threadNumber = (int)parameters[0];
            int threadCount = (int)parameters[1];

            string dataPath = ConfigurationManager.AppSettings["GeocoderDataPath"].ToString();
            UsaGeocoder usaGeocoder = new UsaGeocoder(dataPath);
            usaGeocoder.Open();

            for (int i = 0; i < testDataRecords.Count; i++)
            {
                string item = testDataRecords[i];
                if (i % threadCount != threadNumber)
                {
                    continue;
                }
                string[] parts = item.Split(',');
                string point = string.Format("{0} {1}", parts[10], parts[9]);
                if (string.IsNullOrEmpty(point) || (parts[1] == "NULL"))
                {
                    continue;
                }
                Collection<GeocoderMatch> result = usaGeocoder.Match(point);
            }

            stopWatch.Stop();
            Trace.WriteLine(string.Format("Reverse Geocoder: Thread {0}, Time: {1}s", threadNumber, stopWatch.ElapsedMilliseconds / 1000));
            usaGeocoder.Close();

            GC.Collect();
            GC.SuppressFinalize(this);
            GC.Collect();
        }

        private void btnLoadData_Click(object sender, EventArgs e)
        {
            string testDataPath = txtTestFile.Text;
            StreamReader sr = new StreamReader(testDataPath);
            testDataRecords.Clear();
            while (!sr.EndOfStream)
            {
                testDataRecords.Add(sr.ReadLine());
            }
            sr.Close();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            GC.Collect();
            GC.SuppressFinalize(this);
            GC.Collect();
        }
    }
}
